# epsutil v0.1: graphical Epson ink printer toolbox based on escputil
#
# Copyright 2002 Hans-Peter Jansen <hpj@urpla.net>
#
# This program is placed under the GNU General Public License V.2

import os
import re
import posix
import pwd

cfgdir = os.path.expanduser('~')            # config base dir
cfgfile = os.path.join(cfgdir, '.epsutil')  # ascii config file

user = pwd.getpwuid(posix.getuid())[0]

version = '0.1'
program = 'EpsUtil'

isfloat = re.compile('([0-9]+\.[0-9]+)')
isint = re.compile('([0-9]+)')

LISTQUOTE = "'"
LISTSEP = ','
APPLYSEP = ' = '
HOMESYM = '~'

home = os.path.expanduser(HOMESYM)

def strcmp(str, substr):
    l = len(substr)
    return len(str) >= l and str[:l] == substr

class Config:
    """ members of this class are saved to cfgdir/cfgfile
        this handles tree basic types and one list type:
        - strings are quoted, but parsing is relaxed
          must not contain LISTSEP
          preceding '~' is expanded/compressed to user home dir
        - ints
        - floats
        - lists of the above
        when adding persistant items, make sure to put them in keys, too
        (it's used for indexing/ordering output)
    """
    def __init__(self):
        self.cfgFile = cfgfile      # RO: change variables instead
        self.cfgVersion = version   # RO
        #self.style = 'platinum'
        self.style = ''
        self.saveLayout = 1
        self.mainScreenSize = [148, 366]
        self.saveLocation = 1
        self.mainScreenLocation = []
        self.devices = ['/dev/lp0', '/dev/lp1', '/dev/lp2', 
                        '/dev/usb/lp0', '/dev/usb/lp1', '/dev/usb/lp2']
        self.devicesIdx = 3
        self.model = ''

    def keys(self):
        """ this tuple serves two purposes:
            - save order
            - persistent keys
              in order to allow runtime config values
        """
        return (
            'cfgFile', 'cfgVersion', 'style',
            'saveLayout', 'mainScreenSize',
            'saveLocation', 'mainScreenLocation',
            'devices', 'devicesIdx', 'model'
               )

    def typequote(self, v):
        """ quote value according to type """
        if type(v) == type(''):
            if strcmp(v, home):
                v = HOMESYM + v[len(home):]
            v = "%s%s%s" % (LISTQUOTE, v, LISTQUOTE)
        else:
            v = str(v)
        return v

    def typecast(self, v):
        """ cast to matching type """
        if v:
            # quoted string type
            if v[0] == LISTQUOTE and v[-1] == LISTQUOTE:
                # unquote string
                v = v[1:-1]
            else:
                # check float
                m = isfloat.search(v)
                if m:
                    v = float(m.group(1))
                else:
                    m = isint.search(v)
                    if m:
                        v = int(m.group(1))
        if type(v) == type('') and strcmp(v, HOMESYM):
            v = os.path.expanduser(v)
        return v

    def save(self):
        """ save config file """
        cfg = "# %s configuration V%s\n" % (program, version)
        for k in self.keys():
            if not hasattr(self, k):
                continue
            v = self.__dict__[k]
            #print "k,v:", k, v
            if type(v) == type([]):
                l = v
                v = ''
                for i in l:
                    i = self.typequote(i)
                    if v:
                        v = '%s%s%s' % (v, LISTSEP, i)
                    else:
                        v = i
            else:
                v = self.typequote(v)
            cfg = cfg + "%s%s%s\n" %(k, APPLYSEP, v)
        #print "save:\n", cfg
        if not os.path.exists(cfgdir):
            os.mkdir(cfgdir, 0700)
        f = open(cfgfile, "w")
        f.write(cfg)
        f.close()

    def load(self):
        """ load config file """
        if not os.path.exists(cfgfile):
            return 0
        f = open(cfgfile, "r")
        cfg = f.read()
        f.close()
        #print "load:\n", cfg
        if not cfg:
            return
        for i in cfg.split("\n"):
            if not len(i) or i[0] == '#':
                continue
            k, v = i.split(APPLYSEP)
            if k not in self.__dict__.keys():
                continue
            t = self.__dict__[k]
            if type(t) == type([]):
                l = v
                v = []
                for i in l.split(LISTSEP):
                    i = self.typecast(i)
                    v.append(i)
            else:
                v = self.typecast(v)
            #exec("self.%s = v" % k)
            self.__dict__[k] = v
            #print "config: %s%s%s" % (k, APPLYSEP, str(v))
